-- Daily Global Check
-- by Jadya
-- EU-Well of Eternity

local tinsert, tremove, ceil = table.insert, table.remove, math.ceil

local addonName, addonTable = ...
local hbdp = LibStub("HereBeDragons-Pins-1.0")

-- string colors
local T = addonTable.T
local AddColor = addonTable.AddColor

-- zone names cache
DailyGlobalCheck.Z = setmetatable({}, { __index = function(t, k)
												   local result = GetMapNameByID(k) or ""
											       t[k] = result
												   return result
                                                  end})
--[[ map functions ]]--

local lists =  DailyGlobalCheck.ListFunctions
local maps  = DailyGlobalCheck.MapFunctions

local frames_pool = {}
local buttons = { worldmap = { p = CreateFrame("Frame", nil, WorldMapButton), pool = {} },
                  minimap  = { pool = {} } }

local mapcontinent, continents, buttons_by_mapid

-- frame pool
local function getbtn()
 local btn = frames_pool[1]
 if btn then tremove(frames_pool, 1) end
 return btn
end

local function freebtn(btn)
 tinsert(frames_pool, btn)
 btn:Hide()
end
--

-- events
local function mapbutton_onmouseup(self, mousebutton)
 if not IsControlKeyDown() or mousebutton ~= "RightButton" then
  local p = self:GetParent()
  if self:GetParent() == buttons.worldmap.p then
   WorldMapButton_OnClick(WorldMapButton, mousebutton)
  else
   Minimap_OnClick(Minimap, mousebutton)
  end
  return
 end

 local data = DailyGlobalCheck:getQData(self.questID, self.title)

 if data then
  local x = data[5][2] / 100
  local y = data[5][3] / 100
  addonTable.setTomTomWP(data[5][1], nil, x, y, { title = data[2]})
 end
end

local function mapbutton_onenter(self)
 local data = DailyGlobalCheck:getQData(self.questID, self.title)

 if not data then return end

 DailyGlobalCheck_maptooltip:SetOwner(self,"ANCHOR_BOTTOM")
 DailyGlobalCheck_maptooltip:ClearLines()

 DailyGlobalCheck_maptooltip:AddLine("|cff00AAFFDaily Global Check|r")
 DailyGlobalCheck_maptooltip:AddLine(AddColor(self.title or "","LIGHT_BLUE"))
 DailyGlobalCheck_maptooltip:AddLine(data[3].." "..data[2])
 DailyGlobalCheck_maptooltip:AddLine(data[4])
 DailyGlobalCheck_maptooltip:Show()
 
  DailyGlobalCheck_maptooltip.info:SetOwner(DailyGlobalCheck_maptooltip, "ANCHOR_BOTTOM")
 local info = DailyGlobalCheck.QuestFunctions:GetOverlayInfo(self.questID, data)
 if info and DailyGlobalCheck:OverlayInfoToTooltip(DailyGlobalCheck_maptooltip.info, info) then
  DailyGlobalCheck_maptooltip.info:Show()
 else
  DailyGlobalCheck_maptooltip.info:Hide()
 end
end

local function mapbutton_onleave()
 DailyGlobalCheck_maptooltip:Hide()
end
--

--- WorldMap and Minimap functions ---
local function createButton(t, questID, list)

 local f = getbtn()
 
 if not f then
  f = CreateFrame("FRAME")
  f:SetScript("OnMouseUp", mapbutton_onmouseup) 
  f:SetScript("OnEnter", mapbutton_onenter)
  f:SetScript("OnLeave", mapbutton_onleave)
  f:EnableMouse(DailyGlobalCheck_Options.UI[t.."_icon_interact"])
  f.tex = f:CreateTexture()
  f.tex:SetAllPoints()
 end
 
 f:SetParent(t == "worldmap" and buttons[t].p or nil)
 f.questID = questID
 f.title   = list and list.Title
 if not buttons[t].pool[questID] then buttons[t].pool[questID] = {} end
 tinsert(buttons[t].pool[questID], f)
 return f
end

function maps:AddButton(t, questID, plugin, forced)
 local list = type(plugin) == "string" and select(2, lists:Find(plugin)) or plugin
 local v = DailyGlobalCheck:getQData(questID, list and list.Title)

 if v and v[5] then
  local m = v[5]

  if DailyGlobalCheck.Z[m[1]] == "" then return end

  local c = mapcontinent[m[1]]

  -- create world map buttons only if the continent has been seen in this session
  -- otherwise store the info
  if t == "worldmap" and c and not forced then
   if not continents[c] then continents[c] = {} end
   if not continents[c].initialized then
    tinsert(continents[c], { questID, list.Title })
    return
   end
  end

  local btns = buttons[t].pool[questID]
  
  if not btns then
   buttons[t].pool[questID] = {}
   btns = buttons[t].pool[questID]
  end

  local i = 1

  while m[i] do
   local f = btns[ceil(i / 4)] or createButton(t, questID, list)

   if not buttons_by_mapid[m[i]] then buttons_by_mapid[m[i]] = {} end
   if not tContains(buttons_by_mapid[m[i]], f) then
    tinsert(buttons_by_mapid[m[i]], f)
   end
   
   local x = m[i + 1]
   local y = m[i + 2]
   local size = DailyGlobalCheck_Options.UI[t.."_icon_size"]
   f:SetSize(size, size)
   
   local plugin = list or select(2, lists:Find(v[11]))
   f.tex:SetTexture(v[8] or ((plugin and plugin.Icon) or T.dgcicon), 1)
   if t == "minimap" then
    hbdp:AddMinimapIconMF("DailyGlobalCheck", f, m[i], nil, x / 100, y / 100)
   else
    hbdp:AddWorldMapIconMF("DailyGlobalCheck", f, m[i], nil, x / 100, y / 100)
   end
  i = i + 4
 end
 end
end

function maps:AddList(t, list)
 if not lists:isList(list) then return end
 
  for _, page in pairs(list.Order) do
   for _, group in pairs(page) do
    for i = 2, #group do
	 local questID = DailyGlobalCheck.GetRealQuestID(group[i], list.Title)
	 if not DailyGlobalCheck.isquestcompleted(group[i], true, list) and addonTable.isshown(group[i]) then
      maps:AddButton(t, questID, list)
	 else
	  maps:RemoveButton(t, questID)
	 end
	end
   end
  end
end

function maps:RemoveButton(t, questID)
 local btns = buttons[t].pool[questID]

 if btns then
  for _,v in pairs(btns) do
   freebtn(v)
   if t == "minimap" then
    hbdp:RemoveMinimapIcon("DailyGlobalCheck", v)
   else
    hbdp:RemoveWorldMapIcon("DailyGlobalCheck", v)
   end
  end
  buttons[t].pool[questID] = nil
 end
end

function maps:RemoveList(t, list)
 if not lists:isList(list) then return end
 
 for _, page in pairs(list.Order) do
  for _, group in pairs(page) do
   for i = 2, #group do
    local questID = DailyGlobalCheck.GetRealQuestID(group[i], list.Title)
    maps:RemoveButton(t, questID)
   end
  end
 end
end
---------------------------------

function maps:UpdateList(t, list)
 if (t == "worldmap" and not DailyGlobalCheck_Options["show_worldmapicons"]) or 
    (t == "minimap" and  not DailyGlobalCheck_Options["show_minimapicons"]) then
  return
 end

 if lists:isActive(list) and lists:getShowMapIcons(list, t) then
  maps:AddList(t, list)
 else
  maps:RemoveList(t, list)
 end
end

function maps:UpdateIcons(t, list)

 if not list and t == "worldmap" and not DailyGlobalCheck_Options["show_worldmapicons"] then
  hbdp:RemoveAllWorldMapIcons("DailyGlobalCheck")
  return
 end

 if not list and t == "minimap" and not DailyGlobalCheck_Options["show_minimapicons"] then
  hbdp:RemoveAllMinimapIcons("DailyGlobalCheck")
  return
 end

 if list and lists:isList(list) then
  maps:UpdateList(t, list)
 else
  DailyGlobalCheck:foreachLists(function(_,v)
   maps:UpdateList(t, v)
  end, false)
 end
end

function maps:UpdateIconsSize(t)
 if not buttons[t] then return end
 
 local size = DailyGlobalCheck_Options.UI[t.."_icon_size"]
 for _,v in pairs(buttons[t].pool) do
  for _,btn in pairs(v) do
   btn:SetSize(size, size)
  end
 end
end

function maps:UpdateInteractable(t)
 if not buttons[t] then return end

 for _,v in pairs(buttons[t].pool) do
  for _,btn in pairs(v) do
   btn:EnableMouse(DailyGlobalCheck_Options.UI[t.."_icon_interact"])
  end
 end
end

function maps:UpdateQuestID(questID, list)
 local new_questID = list.."<!"..questID
 if buttons["worldmap"].pool[questID] then
  buttons["worldmap"].pool[new_questID] = buttons["worldmap"].pool[questID]
  buttons["worldmap"].pool[questID] = nil
 end
 
 for _,v in pairs(continents) do
  for i = 1, #v do
   if tostring(v[i][1]) == tostring(questID) then
    v[i][1] = new_questID
    break
   end
  end
 end
end

local function checkMap(v)
 if not v then return end
 
 local i = 1
 while i <= #v do
  if DailyGlobalCheck.isquestcompleted(v[i].questID, true) then
   maps:RemoveButton("worldmap", v[i].questID)
   maps:RemoveButton("minimap", v[i].questID)
   tremove(v, i)
  else
   i = i + 1
  end
 end
end

local function initializeContinent(c)
 for i = 1, #continents[c] do
  maps:AddButton("worldmap", continents[c][i][1], continents[c][i][2], true)
 end

 maps:UpdateIcons("worldmap")

 wipe(continents[c])
 continents[c].initialized = true
end

function maps:Initialize()
 CreateFrame("GameTooltip","DailyGlobalCheck_maptooltip", nil, "GameTooltipTemplate")
 DailyGlobalCheck_maptooltip:SetFrameStrata("TOOLTIP")
 DailyGlobalCheck_maptooltip:SetScale(0.8)
 DailyGlobalCheck_maptooltip.info = CreateFrame("GameTooltip", "DailyGlobalCheck_maptooltipInfo", DailyGlobalCheck_maptooltip, "GameTooltipTemplate")
 DailyGlobalCheck_maptooltip.info:SetScale(0.9)

 buttons_by_mapid = {}
 continents       = {}
 mapcontinent     = {}

 local function fillContinentData(c, ...)
  if not ... then return end
  
  local length = select("#", ...)
  
  for i = 1, length, 2 do
   local mapID = select(i, ...)
   mapcontinent[mapID] = c
  end
 end
 
 for i = 1, 8 do
  fillContinentData(i, GetMapZones(i))
 end

 -- garrisons
 mapcontinent[971] = 7
 mapcontinent[976] = 7

 DGCEventFrame:RegisterEvent("WORLD_MAP_UPDATE")
 local last_check = 0
 function DGCEventFrame.WORLD_MAP_UPDATE(...)
  if InCombatLockdown() then return end
  
  local map = GetCurrentMapAreaID()
  local c   = GetCurrentMapContinent()
 
  if continents[c] and not continents[c].initialized then
   initializeContinent(c)
  end

   -- overall check every 2~ minutes to be sure we hide completed quests' icons
  local now = time()
  if now > last_check + 120 then
   for k,v in pairs(buttons_by_mapid) do
    checkMap(v.questID)
   end
   last_check = now
   return
  end

  -- minimal check only for the current zone with every map update
  local v = map and buttons_by_mapid[map] or nil
  checkMap(v)
 end
end